//Dan Folescu 2021
#ifndef __CALCULATIONS_H__
#define __CALCULATIONS_H__

#include "constants.h"
#include "structures.h"

/* estimates the molecules "electrostatic radius" - see Sigalov 2006 */
double estimate_A(residue**,int);

/* calculates electric field normal for a single vertex */
void calculate_electric_field_normal_vertex(vertex*,physical_structure*,model_parameters*);

/* calculates electric field normal for a single triangle */
void calculate_electric_field_normal_triangle(triangle*);

/* calculates electric field normal for the entire molecule */
void calculate_electric_field_normal_molecule(physical_structure*,triangulation_structure*,model_parameters*);

/* calculates electric field normal for the given residue list */
void calculate_electric_field_normal_residue_list(residue**,physical_structure*,triangulation_structure*,model_parameters*);

/* calculates molecular surface charge density */
void calculate_surface_charge_density_molecule(physical_structure*, triangulation_structure*);

/* calculates surface charge density for the given residue list */
void calculate_surface_charge_density_residue_list(residue*, physical_structure*, triangulation_structure*);

/* calculates the total surface charge of the entire molecule */
double calculate_total_surface_charge(triangulation_structure*);

/* calculate the molecule's solvation energy */
double compute_solvation_energy(physical_structure*, triangulation_structure*);

/* VECTOR MATH */

/* dot product for 2 3-d vectors */
double dot_prod   (double v1[3], double v2[3]);

/* magnitude of a vector (dist of vector with origin */
double magnitude  (double v[3]);

/* cross product between two vectors, gets stored in cross */
void cross_prod (double v1[3], double v2[3], double cross[3]);

/* distance between 2 3d points */
double dist (double x, double y, double z, double x2, double y2, double z2);

/* distance between 2 3d points squared */
double dist2(double x, double y, double z, double x2, double y2, double z2);

static int cmpfunc (const void * a, const void * b) {
  double num1 = *(double*)a;
  double num2 = *(double*)b;

  return num1 >= num2 ? 1 : -1;
}

/* MACROS */

/* returns the minimum, a or b */
#define min(a, b) (a < b)?a:b

/* returns the maximum, a or b */
#define max(a, b) (a > b)?a:b

/* clamps a between b and c */
#define clamp(a, b, c) (max(b, (min(a, c))))

#endif
